%Generate Mie Lookup Table
function Qf = MieKernal(D,mR,mI,Lmda_s,Lmda_a)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following code is used to generate a dry scattering and absorption
% efficiency lookup table (i.e., Qf). This code requires the set of Mie
% forward model MATLAB functions (C. Mätzler May, 2002) which was
% retrieved from https://omlc.org/software/mie/ on January 1st, 2019.
% Each of the required inputs for this code are described as follows:
% 1) D: The mid-point diameters for each size resolved number 
%   concentration (i.e., n_o) channel.
%    -> dimensions: [number of n_o channels,1]
%    -> Units: nm
% 2) mR: Ascending column vector of desired search range of the real
%    portion of the m_d,t (i.e., n_d,t). 
%    -> dimensions: [1,number of mR increments]
%    -> EXAPMLE: 1.33:0.01:1.70
%    -> Units: unitless
% 3) mI: Ascending column vector of desired search range of the imaginary 
%    portion of the m_d,t (i.e., k_d,t). 
%    -> dimensions: [1,number of mI increments]
%    -> EXAPMLE: 0:0.01:0.1
%    -> Units: unitless
% 4) Lmda_s: Wavelengths that correspond to the 3 dry scattering
%    coefficient measurement channels.
%    -> dimensions: [1,3]
%    -> Units: nm
% 5) Lmda_a: Wavelengths that correspond to the 3 dry absorption 
%    coefficient measurement channels.
%    -> dimensions: [1,3]
%    -> Units: nm
% The output of this code is Qf, which is a multidimensional matrix that
% is described as follows:
%    -> dimensions: [number of mR increments, number of mI increments,
%                      number of n_o channels,6]
%    -> Units: unitless
% *Designed for MATLAB 2021a and requires the parallel computing MATLAB 
% toolbox*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
% combine Lmda_s and Lmda_a to reduce number of Qf dimensions
wvl = [Lmda_s,Lmda_a];
 
% determine the length of each dimension of Qf
L1 = length(mR);L2 = length(mI);L3 = length(D);L4 = length(wvl);
 
% initialize Qf
Qf = nan(L3,L1,L2,L4);
 
% Iterate through every position of Qf to calculate each value of the
% lookup table using Mie.m
parfor i=1:L1
    dp = D;WVL = wvl;
    mr = mR(i);
    mii = mI;
    for j=1:L2
        mi = mii(j);
        for g = 1:L4
            for k=1:L3
                aio = Mie(complex(mr,mi),pi*dp(k)/WVL(g));
                if g <= length(Lmda_s)
                    Qf(k,i,j,g) = real(aio(2));
                else
                    Qf(k,i,j,g) = real(aio(3));
                end
                
            end
        end
    end
end
end
