%Generate Covariance Matrix
function covM = S_e(y,Acur,sig1,cFct)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following code is used to generate the covariance matrix (covM) of 6
% optical coefficient measurements that are correlated by the correlation
% factor (i.e., cr). Each of the required inputs for this code are
% described as follows:
% 1) y: 6 measured optical coefficients
%    -> dimensions: [1, 6]
%    -> Units: # Mm-1
% 2) Acur: 3 Scattering and 3 absorption measurement accuracy values used
%    to calculate Chi'.
%    -> dimensions: [1, 6]
%    -> FORMAT: [accuracy of scattering channels 1 to 3, accuracy of 
%                absorption channels 1 to 3]
%    -> EXAPMLE: [0.1, 0.1, 0.1, 0.05, 0.05, 0.05]
%    -> Units: unitless
% 3) Sig1: 3 Scattering and 3 absorption measurement precision values
%    divided by the square root of the data resolution (i.e., number of
%    native optical measurements averaged to create data set). 
%    -> dimensions: [1, 6] 
%    -> FORMAT: [precision of scattering channels 1 to 3, precision of
%                absorption channels 1 to 3] 
%    -> Units: Mm-1
% 4) cFct: Optical measurement correlation factor.
%    -> dimensions: [1, 1]
%    -> EXAPMLE: 0.3
%    -> Units: unitless
% The output of this code is covM that is described as follows:
%    -> dimensions: [6, 6]
%    -> Units: mM-1
% *Designed for MATLAB 2021a*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
% define the diagonal positions of covM
covm = [y(1)*Acur(1)+sig1(1);y(2)*Acur(2)+sig1(2);y(3)*Acur(3)+sig1(3);...
    y(4)*Acur(4)+sig1(4);y(5)*Acur(5)+sig1(5);y(6)*Acur(6)+sig1(6)];
 
% initialize the loop to calculate the upper and lower triangle of covM
% based off the diagonal values
covM = nan(length(covm));
 
% calculate the non-diagonal positions of covM
for io = 1:size(covm,1)
    for io2 = 1:size(covm,1)
        if io2 == io
            covM(io,io2) = covm(io);
        else
            covM(io,io2) = cFct*sqrt(covm(io))*sqrt(covm(io2));
        end
    end
end
end
