%Retrieve Particle Density
function rho = cal_rho(no_las,D_las,no_aps,D_aps)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following code is used to compute the density (i.e., rho) of the
% particles measured within the overlapping measurement region of the
% aerodynamic particle sizer (APS) and the laser aerosol spectrometer
% (LAS). The following single dimension vectors are required as inputs for
% this code:
% 1) no_las: The lognormal size resolved number concentrations measured 
%    by the LAS that overlap the native APS size range. 
% 2) D_las: The LAS mid-point diameters that overlap the native APS size 
%    range.
% 3) no_aps: The lognormal size resolved number concentration measured 
%   by the APS. 
% 4) D_aps: The APS mid-point diameters. 
%   The output of this code is a double value of the retrieved rho.
% *Designed for MATLAB 2021a and requires the statistics and machine
% learning MATLAB toolboxes*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
% define density space
ro = 0.9:0.1:3.0;
 
% filter for NAN values in the LAS number concentration
ihu = find(~isnan(no_las));
n_LAS = no_las(ihu);DLAS = D_las(ihu);
 
% find coarse mode median optical diameter within the overlap range
JJ = find(n_LAS==max(n_LAS));
n_las = n_LAS(JJ:end);Dlas = DLAS(JJ:end);
 
% check to ensure there is enough LAS number concentration data to perform
% the fitting process
if sum(~isnan(n_las))>3 & sum(~isnan(no_aps))>1
    % try fitting the N_las to an exponential decay model using the 
    % starting values prescribed by P. A. Baron and K. Willeke, (2011)
    beta0 = [24 3.08];
    try
        mdl_las = fitnlm(Dlas,n_las',pwrfn,beta0);
    catch
        try
           % if initial attempt faily try fitting using a smaller value for
           % C1 that was presribed by P. A. Baron and K. Willeke, (2011)
            beta0 = [0.24 3.08];
            mdl_las = fitnlm(Dlas,n_las',pwrfn,beta0);
        catch
            % if both attempts fail. result is nan;
            mdl_las = nan;
        end
    end
    % once the LAS model parameters are determined, the median aerodynamic
    % dynameter is determined by searching for the maximum number
    % concentrations measured by the APS
    s2 = nan(length(ro),1);
    jj = find(no_aps==max(no_aps));
    
    % compute the objective function for the entire range of density values.
    for i2 = 1:length(ro)
        if jj<length(no_aps)
            DAPS = D_aps(jj:end)./sqrt(ro(i2))/1000;
            naps = no_aps(jj:end);
            jij = find(DAPS>=Dlas(1)&DAPS<=Dlas(end));
            d_aps = DAPS(jij);n_aps = (naps(jij));
            dlogdn_las = mdl_las.Coefficients.Estimate(1).*d_aps.^( ...
                -mdl_las.Coefficients.Estimate(2));
            s2(i2) = ((length(d_aps))^(-1))*sum((log(dlogdn_las')- ...
                log(n_aps)).^2,'all');
        end
    end
    
    % determine if any of the objective functions were not infinite or nan
    % and minimize the objective function
    if length(~isnan(s2))>1
        s2(isinf(s2))=nan;
        jj2 = find(s2==min(s2,[],'all','omitnan'));
        if ~isempty(jj2)
            % Output
            rho = ro(jj2);
        end
    end
end
end
