%Fit Particle Size Distribution
function op = fit_SD(n_o,N1,N2,N3,Nc,D)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code iteratively fits a measured total size resolved lognormal
% aerosol concentration (i.e., n_o) to a multimodal lognormal model and
% outputs the fitted modal number concentration (i.e., Nj), geometric
% standard deviation (i.e., sgj), and geometric mean diameter (i.e., Dgj).
% where. This code was created using the outline and guidelines provided by
% Hussein et al. (2005). Each of the required inputs for this code are
% described as follows: 
% 1) n_o: Measured lognormal size resolved number concentration.
%    -> dimensions: [1, number of measurement channels]
%    -> Units: # cm-3
% 2) N1: The number concentration of particles with diameters between 7 and
%    20 nm.
%    -> dimensions: [1,1]
%    -> Units: # cm-3
% 3) N2: The number concentration of particles with diameters between 35
%    and 80 nm.
%    -> dimensions: [1,1]
%    -> Units: # cm-3
% 4) N3: The number concentration of particles with diameters between 90
%    and 500 nm.
%    -> dimensions: [1,1]
%    -> Units: # cm-3
% 5) Nc: The number concentration of particles with diameters greater than
%    900 nm.
%    -> dimensions: [1,1]
%    -> Units: # cm-3
% 6) D: The n_o measurement channels' mid-point diameters.
%    -> dimensions: [number of no measurement channels,1]
%    -> Units: nm
% The output of this code is op, which is a MATLAB table containing 12
% columns for each of the potential modal parameters (e.g., Nj, sgj, and
% Dgj) and the corresponding goodness of fit parameters (i.e., R and
% p-value). Where j = 1:4 for the nucleation, Aitken, Accumulation, and
% coarse modes. NOTE: This algorithm will determine the optimal number of
% modes and some modes may be removed during the fitting, which will
% result in NAN values being place for the fit parameters of the mode that
% is removed.
%    -> dimensions: [1,12]
%    -> Units: [# cm-3, unitless, nm, # cm-3, unitless, nm, 
%       # cm-3, unitless, nm, # cm-3, unitless, nm, unitless, unitless]
% *Designed for MATLAB 2021a and requires the statistics and machine
% learning MATLAB toolbox*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
% initialize and set fitting options.
sd_param = nan(1,12);
sd_gof = nan(1,2);
opts = optimoptions('lsqcurvefit','Algorithm','trust-region-reflective',...
    'Display','off','FunctionTolerance',1e-24,'StepTolerance',1e-24,...
    'OptimalityTolerance',1e-24);
 
% bounded limits for constraining the fit parameters during iterative
% solving steps
acy = [0.95 1.0 1.05];
 
% replace 0 values with NAN. NOTE: if one or more of these values is 0 or
% NAN then the failsafe fitting pathway is followed
N1(N1==0)=nan;N2(N2==0)=nan;N3(N3==0)=nan;Nc(Nc==0)=nan;
 
Nt = [N1 N2 N3 Nc];
 
% filter for NAN values n_o and corresponding D prior to fitting
ik = find(~isnan(n_o));dndlogD = n_o(ik);Dp = D(ik);
 
% if none of the fitting steps are successful, change the start point of
% the fitting values and repeat
stp_fct = [1.0 0.75 1.25];
for i6 = 1:length(stp_fct)
    % define all bounds and starting points of the failsafe pathway and
    % define the max limit for Dgc for the entire algorithm
    UB = max(Dp);
    lb0 = [1.20 1.20 1.20 1.20];sp0 = [1.60 1.60 1.60 1.70]*stp_fct(i6); 
        ub0 = [2.10 2.10 2.10 2.24];
    lb00 = [7. 35 100 900];sp00 = [10 50 250 2500]*stp_fct(i6);
        ub00 = [20 80 500 UB];
    lb01 = [0.5 0.8 0.5 10^(-9)];sp01 = [1.0 1.0 1.0 1]*stp_fct(i6);
        ub01 = [1.1 1.5 1.1 1.5];
    
    % find modes of Nt not measured, and if all 4 modes are measured, begin
    % the normal fitting pathway, else determine number of modes measured
    % and begin basic iterative fitting
    ijo = find(~isnan(Nt(1,:)));
    ACo = Nt(1,ijo);
    if length(~isnan(dndlogD))>12 && length(ACo)==4
        
        % initialize the first fitting iteration, define the surface area
        % model, set the lower and upper bounds, and set the starting
        % points
        mdca = [];ra = [];pa = [];
        r = nan(3,1);p = nan(3,1);
        fourmodefun = @(b,x)(2.203*pi*x.^2).*((b(1)./((sqrt(2*pi))* ...
            log(b(2)))).*exp(-((log(x)-log(b(3))).^2./(2*log(b(2)).^2)))+ ...
            b(4)./((sqrt(2*pi)).*log(b(5))).*exp(-((log(x)-log(b(6)) ...
            ).^2./(2*log(b(5)).^2)))+b(7)./((sqrt(2*pi)).*log(b(8)) ...
            ).*exp(-((log(x)-log(b(9))).^2./(2*log(b(8)).^2)))+ ...
            b(10)./((sqrt(2*pi)).*log(b(11))).*exp(-((log(x)-log(b(12))...
            ).^2./(2*log(b(11)).^2))));
        lb = [0.5*N1(1) 1.20 7. 0.8*N2(1) 1.20 35 0.5*N3(1) 1.20 100 ...
            (10^(-9))*Nc(1) 1.20 900];
        sp = [1.0*N1(1) 1.60 10 1.0*N2(1) 1.60 50 1.0*N3(1) 1.60 250 ...
            (1)*Nc(1) 1.60 2500]*stp_fct(i6);
        ub = [1.1*N1(1) 2.10 20 1.5*N2(1) 2.10 80 1.1*N3(1) 2.10 500 ...
            (1.5)*Nc(1) 2.24 UB];
        
        % fit the 4-mode model using the initial bounds/starting points
        % and determine GOF
        mdl = real(lsqcurvefit(fourmodefun,sp,Dp,(pi*Dp.^2).*dndlogD,...
            lb,ub,opts));
        [ra(1),pa(1)] = corr((pi*Dp'.^2).*dndlogD',fourmodefun(mdl,Dp'));
        mdca(1,:) = mdl;
        
        % Initialize 2nd stage of fitting process by keeping the Nj
        % values fixed, and set the new bounds/starting points
        fourmodefun_2 = @(b,x)(2.203*pi*x.^2).*((mdl(1)/((sqrt(2*pi))* ...
            log(b(1)))).*exp(-((log(x)-log(b(2))).^2./(2*log(b(1)).^2)))+...
            mdl(4)/((sqrt(2*pi)).*log(b(3))).*exp(-((log(x)-log(b(4)) ...
            ).^2./(2*log(b(3)).^2)))+mdl(7)/((sqrt(2*pi)).*log(b(5)) ...
            ).*exp(-((log(x)-log(b(6))).^2./(2*log(b(5)).^2)))+mdl(10)./ ...
            ((sqrt(2*pi)).*log(b(7))).*exp(-((log(x)-log(b(8))).^2./ ...
            (2*log(b(7)).^2))));
        lb2 = [mdl(2) mdl(3) mdl(5) mdl(6) mdl(8) mdl(9) mdl(11) mdl(12) ...
            ]*acy(1);
        sp2 = [mdl(2) mdl(3) mdl(5) mdl(6) mdl(8) mdl(9) mdl(11) mdl(12) ...
            ]*acy(2);
        ub2 = [mdl(2) mdl(3) mdl(5) mdl(6) mdl(8) mdl(9) mdl(11) mdl(12)...
            ]*acy(3);
        
        % fit the 2nd 4-mode model using the new bounds/starting points
        % and determine GOF
        mdl2 = real(lsqcurvefit(fourmodefun_2,sp2,Dp,(pi*Dp.^2).*dndlogD ...
            ,lb2,ub2,opts));
        [ra(2),pa(2)] = corr((pi*Dp'.^2).*dndlogD',fourmodefun_2(mdl2,Dp)');
        mdca(2,:) = [mdl(1) mdl2(1:2) mdl(4) mdl2(3:4) mdl(7) mdl2(5:6) ...
            mdl(10) mdl2(7:8)];
        
        % Initialize 3rd stage of fitting process by keeping the sgj and
        % Dgj values fixed, and set the new bounds/starting points
        fourmodefun_3 = @(b,x)(2.203*pi*x.^2).*((b(1)/((sqrt(2*pi))*log( ...
            mdl2(1)))).*exp(-((log(x)-log(mdl2(2))).^2./(2*log(mdl2(1) ...
            ).^2)))+(b(2))/((sqrt(2*pi)).*log(mdl2(3))).*exp(-((log(x)- ...
            log(mdl2(4))).^2./(2*log(mdl2(3)).^2)))+ b(3)/((sqrt(2*pi) ...
            ).*log(mdl2(5))).*exp(-((log(x)-log(mdl2(6))).^2./(2*log( ...
            mdl2(5)).^2) ))+b(4)/((sqrt(2*pi)).*log(mdl2(7))).*exp(-(( ...
            log(x)-log(mdl2(8))).^2./(2*log(mdl2(7)).^2))));
        lb3 = mdl([1 4 7 10])*acy(1);sp3 = mdl([1 4 7 10])*acy(2);
        ub3 = mdl([1 4 7 10])*(acy(3));
        
        % fit the 3rd 4-mode model using the new bounds/starting points
        % and determine GOF
        mdl3 = real(lsqcurvefit(fourmodefun_3,sp3,Dp,(pi*Dp.^2).*dndlogD,...
            lb3,ub3,opts));
        [ra(3),pa(3)] = corr((pi*Dp'.^2).*dndlogD',(fourmodefun_3(mdl3,Dp')));
        mdca(3,:) = [mdl3(1) mdl2(1:2) mdl3(2) mdl2(3:4) mdl3(3) ...
            mdl2(5:6) mdl3(4) mdl2(7:8)];
        
        % find the 4-mode model that has the highest R and p-value to carry
        % forward for the mode optimization step.
        kji = find(ra == max(ra,[],'all','omitnan'));
        r(1)=ra(kji(end));p(1)=pa(kji(end));mdc = mdca(kji(end),:);
        
        % check for overlap between Dg1 and Dg2
        mdca = [];ra = [];pa = [];
        ap = 0;
        nt = sum(mdl3,'all');
        if abs(log(mdc(3)-mdc(6)))<0.24 & mdc(9)>100
            ap = 1;
        elseif abs(log(mdc(3)-mdc(6)))<0.33 | abs(log(mdc(9)-mdc(6)))< 
                ...0.33 & mdc(9)>100
            ap = 1;
        elseif abs(log(mdc(3)-mdc(6)))<0.33 & mdc(1)/(nt)<0.25 | ...
                mdc(4)/(nt)<0.25
            ap = 1;
        elseif abs(log(mdc(3)-mdc(6)))<0.5 | abs(log(mdc(9)-mdc(6)))<0.5 ...
                & mdc(4)/(nt)<0.25
            ap = 1;
        end
        
        % if overlap between Dg1 and Dg2 is indicated, fit a 3-mode model
        % with only one mode possible between 7 and 100 nm
        if ap == 1
            
            % define 3-mode function and initial bounds/starting points
            threemodefun = @(b,x)(2.203*pi*x.^2).*(b(1)/((sqrt(2*pi))* ...
                log(b(2))).*exp(-((log(x)-log(b(3))).^2./(2*log(b(2) ...
                ).^2)))+b(4)/((sqrt(2*pi)).*log(b(5))).*exp(-((log(x)- ...
                log(b(6))).^2./(2*log(b(5)).^2)))+b(7)/((sqrt(2*pi) ...
                ).*log(b(8))).*exp(-((log(x)-log(b(9))).^2./(2*log( ...
                b(8)).^2))));
            lb4 = [0.8*N1(1) 1.20 7. 0.5*(N3(1)+N2(1)) 1.20 100 ...
                (10^(-9))*Nc(1) 1.20 900];
            sp4 = [1.0*N1(1) 1.60 25 1.0*(N3(1)+N2(1)) 1.60 250 ...
                (1)*Nc(1) 1.60 2500]*stp_fct(i6);
            ub4 = [1.1*N1(1) 2.10 80 1.1*(N3(1)+N2(1)) 2.10 900 ...
                (1.5)*Nc(1) 2.24 UB];
            
            % fit the 3-mode model using the initial bounds/starting point
            % and determine GOF
            mdl4 = real(lsqcurvefit(threemodefun,sp4,Dp,...
                (pi*Dp.^2).*dndlogD,lb4,ub4,opts));
            [ra(1),pa(1)] = corr((pi*Dp'.^2).*dndlogD',...
                threemodefun(mdl4,Dp'));
            mdca(1,:) = mdl4;
            
            % Initialize 2nd stage of fitting process by keeping the Nj
            % values fixed, and set the new bounds/starting points
            threemodefun_2 = @(b,x)(2.203*pi*x.^2).*(mdl4(1)/((sqrt(2*pi) ...
                )*log(b(1))).*exp(-((log(x)-log(b(2))).^2./(2*log(b(1) ...
                ).^2)))+mdl4(4)/((sqrt(2*pi)).*log(b(3))).*exp(-(( ...
                log(x)-log(b(4))).^2./(2*log(b(3)).^2)))+mdl4(7)/(( ...
                sqrt(2*pi)).*log(b(5))).*exp(-((log(x)-log(b(6))).^2./ ...
                (2*log(b(5)).^2))));
            lb22 = [mdl4(2) mdl4(3) mdl4(5) mdl4(6) mdl4(8) mdl4(9)]*acy(1);
            sp22 = [mdl4(2) mdl4(3) mdl4(5) mdl4(6) mdl4(8) mdl4(9)]*acy(2);
            ub22 = [mdl4(2) mdl4(3) mdl4(5) mdl4(6) mdl4(8) mdl4(9)]*acy(3);
            
            % fit the 2nd 3-mode model using the new bounds/starting points
            % and determine GOF
            mdl22 = real(lsqcurvefit(threemodefun_2,sp22,Dp, ...
                (pi*Dp.^2).*dndlogD,lb22,ub22,opts));
            [ra(2),pa(2)] = corr((pi*Dp'.^2).*dndlogD', ...
                threemodefun_2(mdl22,Dp'));
            mdca(2,:) = [mdl4(1) mdl22(1:2) mdl4(4) mdl22(3:4) ...
                mdl4(7) mdl22(5:6)];
            
            % Initialize 3rd stage of fitting process by keeping the sgj
            % and Dgj values fixed, and set the new bounds/starting points
            threemodefun_3 = @(b,x)(2.203*pi*x.^2).*(b(1)/((sqrt(2*pi)...
                )*log(mdl22(1))).*exp(-((log(x)-log(mdl22(2))).^2./ ...
                (2*log(mdl22(1)).^2)))+b(2)/((sqrt(2*pi)).*log(mdl22(3)) ...
                ).*exp(-((log(x)-log(mdl22(4))).^2./(2*log(mdl22(3)).^2)...
                ))+b(3)/((sqrt(2*pi)).*log(mdl22(5))).*exp(-((log(x)-log( ...
                mdl22(6))).^2./(2*log(mdl22(5)).^2))));
            lb3 = mdl4([1 4 5])*acy(1);sp3 = mdl4([1 4 5])*acy(2);
            ub3 = mdl4([1 4 5])*acy(3);
            
            % fit the 3rd 3-mode model using the new bounds/starting points
            % and determine GOF
            mdl33 = real(lsqcurvefit(threemodefun_3,sp3,Dp, ...
                (pi*Dp.^2).*dndlogD,lb3,ub3,opts));
            mdca(3,:) = [mdl33(1),mdl22(1:2),mdl33(2),mdl22(3:4), ...
                mdl33(3),mdl22(5:6)];
            [ra(3),pa(3)] = corr((pi*Dp'.^2).*dndlogD', ...
                (threemodefun_3(mdl33,Dp))');
            
            % find the 3-mode model that has the highest R and p-value to
            % carry forward for the final optimization step.
            kji = find(ra == max(ra,[],'all','omitnan'));
            r(2)=ra(kji(end));p(2)=pa(kji(end));mc3 = mdca(kji(end),:);
        end
        
        % check for overlap between Dg2 and Dg1
        if mdc(9)<90
            ap = 2;
        elseif mdc(7)/(nt)<0.03 & mdc(9)<100
            ap = 2;
        elseif abs(log(mdc(6)-mdc(3)))<0.33 | ...
                abs(log(mdc(9)-mdc(6)))<0.33 & mdc(9)<100
            ap = 2;
        elseif abs(log(mdc(9)-mdc(6)))<0.24
            ap = 2;
        end
        
        % if overlap between Dg2 and Dg3 is indicated fit a 3-mode model
        % with two modes between 7 and 100 nm and no mode between 100 and
        % 500 nm
        if ap == 2
            
            % define 3-mode function and initial bounds/starting points
            threemodefun = @(b,x)(2.203*pi*x.^2).*(b(1)/((sqrt(2*pi))* ...
                log(b(2))).*exp(-((log(x)-log(b(3))).^2./(2*log( ...
                b(2)).^2)))+b(4)/((sqrt(2*pi)).*log(b(5))).*exp(-(( ...
                log(x)-log(b(6))).^2./(2*log(b(5)).^2)))+b(7)/((sqrt( ...
                2*pi)).*log(b(8))).*exp(-((log(x)-log(b(9))).^2./(2*log( ...
                b(8)).^2))));
            lb4 = [0.8*(N1(1)+N2(1)) 1.20 7. 0.5*N3(1) 1.20 35 ...
                (10^(-9))*Nc(1) 1.20 900];
            sp4 = [1.0*(N1(1)+N2(1)) 1.60 10 1.0*N3(1) 1.60 50 ...
                (1)*Nc(1) 1.60 2500]*stp_fct(i6);
            ub4 = [1.1*(N1(1)+N2(1)) 2.10 24 1.1*N3(1) 2.10 80 ...
                (1.5)*Nc(1) 2.24 UB];
            mdl4 = real(lsqcurvefit(threemodefun,sp4,Dp, ...
                (pi*Dp.^2).*(dndlogD),lb4,ub4,opts));
            
            % fit the 3-mode model using the initial bounds/starting point
            % and determine GOF
            [ra(1),pa(1)] = corr((pi*Dp'.^2).*dndlogD', ...
                threemodefun(mdl4,Dp'));
            mdca(1,:) = mdl4;
            
            % initialize 2nd stage of fitting process by keeping the Nj
            % values fixed, and set the new bounds/starting points
            threemodefun_2 = @(b,x)(2.203*pi*x.^2).*(mdl4(1)/((sqrt( ...
                2*pi))*log(b(1))).*exp(-((log(x)-log(b(2))).^2./(2*log( ...
                b(1)).^2)))+mdl4(4)/((sqrt(2*pi)).*log(b(3))).*exp(-(( ...
                log(x)-log(b(4))).^2./(2*log(b(3)).^2)))+mdl4(7)/((sqrt( ...
                2*pi)).*log(b(5))).*exp(-((log(x)-log(b(6))).^2./(2*log( ...
                b(5)).^2))));
            lb22 = [mdl4(2) mdl4(3) mdl4(5) mdl4(6) mdl4(8) mdl4(9)]*acy(1);
            sp22 = [mdl4(2) mdl4(3) mdl4(5) mdl4(6) mdl4(8) mdl4(9)]*acy(2);
            ub22 = [mdl4(2) mdl4(3) mdl4(5) mdl4(6) mdl4(8) mdl4(9)]*acy(3);
            
            % fit the 2nd 3-mode model using the new bounds/starting points
            % and determine GOF
            mdl22 = real(lsqcurvefit(threemodefun_2,sp22,Dp,...
                ((pi*Dp.^2).*dndlogD),lb22,ub22,opts));
            [ra(2),pa(2)] = corr((pi*Dp'.^2).*dndlogD',...
                threemodefun_2(mdl22,Dp'));
            mdca(2,:) = [mdl4(1) mdl22(1:2) mdl4(4) mdl22(3:4) ...
                mdl4(7) mdl22(5:6)];
            
            % Initialize 3rd stage of fitting process by keeping the sgj
            % and Dgj values fixed, and set the new bounds/starting points
            threemodefun_3 = @(b,x)(2.203*pi*x.^2).*(b(1)/((sqrt(2*pi) ...
                )*log(mdl22(1))).*exp(-((log(x)-log(mdl22(2))).^2./( ...
                2*log(mdl22(1)).^2)))+b(2)/((sqrt(2*pi)).*log(mdl22(3)) ...
                ).*exp(-((log(x)-log(mdl22(4))).^2./(2*log(mdl22(3)).^2) ...
                ))+b(3)/((sqrt(2*pi)).*log(mdl22(5))).*exp(-((log(x)- ...
                log(mdl22(6))).^2./(2*log(mdl22(5)).^2))));
            lb3 = mdl4([1 4 7])*acy(1);sp3 = mdl4([1 4 7])*acy(2);
            ub3 = mdl4([1 4 7])*acy(3);
            
            % fit the 3rd 3-mode model using the new bounds/starting points
            % and determine GOF
            mdl33 = real(lsqcurvefit(threemodefun_3,sp3,Dp, ...
                ((pi*Dp.^2).*dndlogD),lb3,ub3,opts));
            mdca(3,:) = [mdl33(1),mdl22(1:2),mdl33(2),mdl22(3:4), ...
                mdl33(3),mdl22(5:6)];
            [ra(3),pa(3)] = corr((pi*Dp'.^2).*dndlogD', ...
                (threemodefun_3(mdl33,Dp))');
            
            % find the 3-mode model that has the highest R and p-value to
            % carry forward for the final optimization step
            kji = find(ra == max(ra,[],'all','omitnan'));
            r(2)=ra(kji(end));p(2)=pa(kji(end));mc3 = mdca(kji(end),:);
        end
        
        % define 2-mode function and initial bounds/starting points
        mdca = [];ra = [];pa = [];
        twomodefun = @(b,x)(2.203*pi*x.^2).*(b(1)/((sqrt(2*pi))*log( ...
            b(2))).*exp(-((log(x)-log(b(3))).^2./(2*log(b(2)).^2)))+ ...
            b(4)/((sqrt(2*pi))*log(b(5))).*exp(-((log(x)-log(b(6)) ...
            ).^2./(2*log(b(5)).^2))));
        NT = N1(1)+N2(1)+N3(1);
        lb = [0.9*(NT) 1.20 7.0 (10^(-9))*Nc(1) 1.20 900];
        sp = [1.0*(NT) 1.60 250 (1)*Nc(1) 1.60 2500]*stp_fct(i6);
        ub = [1.1*(NT) 2.10 500 (1.5)*Nc(1) 2.24 UB];
        
        % fit the 2-mode model using the initial bounds/starting point and
        % determine GOF
        mdl0 = real(lsqcurvefit(twomodefun,sp,Dp,...
            ((pi*Dp.^2).*dndlogD),lb,ub,opts));
        [ra(1),pa(1)] = corr((pi*Dp'.^2).*dndlogD',twomodefun(mdl0,Dp)');
        mdca(1,:) = mdl0;
        
        % Initialize 2nd stage of fitting process by keeping the Nj
        % values fixed, and set the new bounds/starting points
        twomodefun_2 = @(b,x)(2.203*pi*x.^2).*((mdl0(1)/((sqrt(2*pi) ...
            )*log(b(1)))).*exp(-((log(x)-log(b(2))).^2./(2*log(b(1)).^2) ...
            ))+(mdl0(2)/((sqrt(2*pi))*log(b(3)))).*exp(-((log(x)-log(b(4) ...
            )).^2./(2*log(b(1)).^2))));
        lb12 = [mdl0(2) mdl0(3) mdl0(5) mdl0(6)]*acy(1);
        sp12 = [mdl0(2) mdl0(3) mdl0(5) mdl0(6)]*acy(2);
        ub12 = [mdl0(2) mdl0(3) mdl0(5) mdl0(6)]*acy(3);
        
        % fit the 2nd 2-mode model using the new bounds/starting points
        % and determine GOF
        mdl12 = real(lsqcurvefit(twomodefun_2,sp12,Dp, ...
            (pi*Dp.^2).*dndlogD,lb12,ub12,opts));
        [ra(2),pa(2)] = corr((pi*Dp'.^2).*dndlogD',twomodefun_2(mdl12,Dp)');
        mdca(2,:) = [mdl0(1) mdl12(1:2) mdl0(2) mdl12(3:4)];
        
        % Initialize 3rd stage of fitting process by keeping the sgj
        % and Dgj values fixed, and set the new bounds/starting points
        twomodefun_3 = @(b,x)(2.203*pi*x.^2).*((b(1)/((sqrt(2*pi))*log( ...
            mdl12(1)))).*exp(-((log(x)-log(mdl12(2))).^2./(2*log( ...
            mdl12(1)).^2)))+(b(2)/((sqrt(2*pi))*log(mdl12(3)))).*exp(...
            -((log(x)-log(mdl12(4))).^2./(2*log(mdl12(3)).^2))));
        lb11 = mdl0([1 4])*acy(1);sp11 = mdl0([1 4])*acy(2);
        ub11 = mdl0([1 4])*acy(3);
        
        % fit the 3rd 2-mode model using the new bounds/starting points
        % and determine GOF
        mdl13 = real(lsqcurvefit(twomodefun_3,sp11,Dp,...
            (pi*Dp.^2).*dndlogD,lb11,ub11,opts));
        [ra(3),pa(3)] = corr((pi*Dp'.^2).*dndlogD',...
            (twomodefun_3(mdl13,Dp)'));
        mdca(3,:) = [mdl13(1) mdl12(1:2) mdl13(2) mdl12(3:4)];
        
        % find the 2-mode model that has the highest R and p-value to
        % carry forward for the final optimization step
        kji = find(ra == max(ra,[],'all','omitnan'));
        r(3)=ra(kji(end));p(3)=pa(kji(end));mc4 = mdca(kji(end),:);
        
        % the final optimization step checks to see which of the three
        % models have the highest R and lowest p-value and subsequently fits
        % the volume-based models while fixing the 3 sets of fine mode
        % parameters
        
        % if the 4-mode surface model has highest R and lowest p-value,
        % proceed with 4-mode volume fitting
        if r(1) == max(r) && r(1) > 0.9 && p(1) < 0.01
            
            % define 4-mode volume model and initial bounds/starting points
            % of the coarse mode
            mdcb = [];rab = [];pab = [];
            m4fun = @(b,x)(2.203*pi*x.^3).*((mdc(1)./((sqrt(2*pi))*log( ...
                mdc(2)))).*exp(-((log(x)-log(mdc(3))).^2./(2*log( ...
                mdc(2)).^2)))+mdc(4)./((sqrt(2*pi)).*log(mdc(5))).*exp( ...
                -((log(x)-log(mdc(6))).^2./(2*log(mdc(5)).^2)))+mdc(7)./( ...
                (sqrt(2*pi)).*log(mdc(8))).*exp(-((log(x)-log(mdc(9)) ...
                ).^2./(2*log(mdc(8)).^2)))+b(1)./((sqrt(2*pi)).*log( ...
                b(2))).*exp(-((log(x)-log(b(3))).^2./(2*log(b(2)).^2))))/6;
            lb = [(10^(-9))*Nc(1) 1.20 900];
            sp = [(1)*Nc(1) 1.60 2500]*stp_fct(i6);
            ub = [(1.5)*Nc(1) 2.24 UB];
            
            % fit the 4-mode volume model using the initial bounds/starting
            % point and determine GOF
            md00 = real(lsqcurvefit(m4fun,sp,Dp, ...
                (pi*Dp.^3).*(dndlogD)/6,lb,ub,opts));
            [rab(1),pab(1)] = corr((pi*Dp'.^3).*dndlogD'/6,m4fun(md00,Dp)');
            mdcb(1,:) = md00;
            
            % Initialize 2nd stage of fitting process by keeping the Nj
            % values fixed, and set the new bounds/starting points
            m4fun_2 = @(b,x)(2.203*pi*x.^3).*((mdc(1)./((sqrt(2*pi))*log( ...
                mdc(2)))).*exp(-((log(x)-log(mdc(3))).^2./(2*log(mdc(2) ...
                ).^2)))+mdc(4)./((sqrt(2*pi)).*log(mdc(5))).*exp(-(( ...
                log(x)-log(mdc(6))).^2./(2*log(mdc(5)).^2)))+mdc(7)./(( ...
                sqrt(2*pi)).*log(mdc(8))).*exp(-((log(x)-log(mdc(9))...
                ).^2./(2*log(mdc(8)).^2)))+md00(1)./((sqrt(2*pi) ...
                ).*log(b(1))).*exp(-((log(x)-log(b(2))).^2./(2*log( ...
                b(1)).^2))))/6;
            lb12 = [md00(2) md00(3)]*acy(1);
            sp12 = [md00(2) md00(3)]*acy(2);
            ub12 = [md00(2) md00(3)]*acy(3);
            
            % fit the 2nd 4-mode volume model using the new bounds/starting
            % points and determine GOF
            mdl1c = real(lsqcurvefit(m4fun_2,sp12,Dp, ...
                (pi*Dp.^3).*(dndlogD)/6,lb12,ub12,opts));
            [rab(2),pab(2)] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                m4fun_2(mdl1c,Dp)');
            mdcb(2,:) = [md00(1) mdl1c];
            
            % Initialize 3rd stage of fitting process by keeping the sgj
            % and Dgj values fixed, and set the new bounds/starting points
            m4fun_3 = @(b,x)(2.203*pi*x.^3).*((mdc(1)./((sqrt(2*pi) ...
                )*log(mdc(2)))).*exp(-((log(x)-log(mdc(3))).^2./( ...
                2*log(mdc(2)).^2)))+mdc(4)./((sqrt(2*pi)).*log(mdc(5)) ...
                ).*exp(-((log(x)-log(mdc(6))).^2./(2*log(mdc(5)).^2)))+...
                mdc(7)./((sqrt(2*pi)).*log(mdc(8))).*exp(-((log(x)-log( ...
                mdc(9))).^2./(2*log(mdc(8)).^2)))+b(1)./((sqrt(2*pi) ...
                ).*log(mdl1c(1))).*exp(-((log(x)-log(mdl1c(2))).^2./( ...
                2*log(mdl1c(1)).^2))))/6;
            lb11 = md00(1)*acy(1);sp11 = md00(1)*acy(2);
ub11 = md00(1)*acy(3);
            
            % fit the 3rd 4-mode volume model using the new bounds/starting
            % points and determine GOF
            mdlc = real(lsqcurvefit(m4fun_3,sp11,Dp, ...
                (pi*Dp.^3).*(dndlogD)/6,lb11,ub11,opts));
            [rab(3),pab(3)] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                m4fun_3(mdlc,Dp'));
            mdcb(3,:) = [mdlc mdl1c];
            
            % find the 4-mode volume model that has the highest R and
            % p-value to carry forward for the final optimization step
            kji = find(rab == max(rab,[],'all','omitnan'));
            ra=rab(kji(end));pa=pab(kji(end));mlc2 = mdcb(kji(end),:);
            
            % determine GOF of 4-mode volume model using orginal 4-mode
            % model Nj, Dgj, and sgj
            [rb,pb] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                fourmodefun_3(mdl3,Dp)');
            
            % if the 4-mode volume model is better save fit parameters
            % else use 4-mode surface model to define output
            if round(ra,5) > round(rb,5)
                sd_param(1,:) = [mdc(1:9) mlc2];
                sd_gof(1,:) = [ra,pa];
            else
                sd_param(1,:) = mdc;
                sd_gof(1,:) = [rb,pb];
            end
        end
        
        % if the 3-mode surface model has the highest (lowest) R (p-value),
        % proceed to fit 3-mode volume model
        if r(2) == max(r) && r(2) > 0.9 && p(2) < 0.01
            
            % define 3-mode volume model and initial bounds/starting points
            % of the coarse mode
            mdcb = [];rab = [];pab = [];
            m4fun = @(b,x)(2.203*pi*x.^3).*((mc3(1)./((sqrt(2*pi))*log( ...
                mc3(2)))).*exp(-((log(x)-log(mc3(3))).^2./(2*log(mc3(2) ...
                ).^2)))+...
                (mc3(4))./((sqrt(2*pi)).*log(mc3(5))).*exp(-((log(x)- ...
                log(mc3(6))).^2./(2*log(mc3(5)).^2)))+b(1)./((sqrt(2*pi) ...
                ).*log(b(2))).*exp(-((log(x)-log(b(3))).^2./(2*log(b(2) ...
                ).^2))))/6;
            lb = [(10^(-9))*Nc(1) 1.20 900];
            sp = [(1)*Nc(1) 1.60 2500]*stp_fct(i6);
            ub = [(1.5)*Nc(1) 2.24 UB];
            
            % fit the 3-mode volume model using the initial bounds/starting
            % point and determine GOF
            md00 = real(lsqcurvefit(m4fun,sp,Dp, ...
                (pi*Dp.^3).*(dndlogD)/6,lb,ub,opts));
            [rab(1),pab(1)] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                m4fun(md00,Dp)');
            mdcb(1,:) = md00;
            
            % Initialize 2nd stage of fitting process by keeping the Nj
            % values fixed, and set the new bounds/starting points
            m4fun_2 = @(b,x)(2.203*pi*x.^3).*((mc3(1)./((sqrt(2*pi))*log( ...
                mc3(2)))).*exp(-((log(x)-log(mc3(3))).^2./(2*log(mc3(2) ...
                ).^2)))+mc3(4)./((sqrt(2*pi)).*log(mc3(5))).*exp(-(( ...
                log(x)-log(mc3(6))).^2./(2*log(mc3(5)).^2)))+md00(1)./(( ...
                sqrt(2*pi)).*log(b(1))).*exp(-((log(x)-log(b(2))).^2./( ...
                2*log(b(1)).^2))))/6;
            lb12 = [md00(2) md00(3)]*acy(1);
            sp12 = [md00(2) md00(3)]*acy(2);
            ub12 = [md00(2) md00(3)]*acy(3);
            
            % fit the 2nd 3-mode volume model using the new bounds/starting
            % points and determine GOF
            mdl1c = real(lsqcurvefit(m4fun_2,sp12,Dp, ...
                (pi*Dp.^3).*(dndlogD)/6,lb12,ub12,opts));
            [rab(2),pab(2)] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                m4fun_2(mdl1c,Dp)');
            mdcb(2,:) = [md00(1) mdl1c];
            
            % Initialize 3rd stage of fitting process by keeping the sgj
            % and Dgj values fixed, and set the new bounds/starting points
            m4fun_3 = @(b,x)(2.203*pi*x.^3).*(mc3(1)./((sqrt(2*pi))*log( ...
                mc3(2))).*exp(-((log(x)-log(mc3(3))).^2./(2*log(mc3(2) ...
                ).^2)))+mc3(4)./((sqrt(2*pi)).*log(mc3(5))).*exp(-((log( ...
                x)-log(mc3(6))).^2./(2*log(mc3(5)).^2)))+b(1)./((sqrt( ...
                2*pi)).*log(mdl1c(1))).*exp(-((log(x)-log(mdl1c(2))).^2./ ...
                (2*log(mdl1c(1)).^2))))/6;
            lb11 = md00(1)*acy(1);sp11 = md00(1)*acy(2);
ub11 = md00(1)*acy(3);
            
            % fit the 3rd 3-mode model using the new bounds/starting points
            % and determine GOF
            mdlc = real(lsqcurvefit(m4fun_3,sp11,Dp, ...
                (pi*Dp.^2).*dndlogD,lb11,ub11,opts));
            [rab(3),pab(3)] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                m4fun_3(mdlc,Dp'));
            mdcb(3,:) = [mdlc mdl1c];
            
            % find the 3-mode volume model that has the highest R and
            % p-value to carry forward for the final optimization step
            kji = find(rab == max(rab,[],'all','omitnan'));
            ra=rab(kji(end));pa=pab(kji(end));mlc2 = mdcb(kji(end),:);
            
            % determine GOF of 3-mode volume model using orginal 3-mode
            % model Nj, Dgj, and sgj
            [rb,pb] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                threemodefun_3(mdl33,Dp'));
            
            % check to see which mode has been removed
            if ap == 1
                akh = [1 0 1 1];
            else
                akh = [1 1 0 1];
            end
            
            % if the volume 3-mode model has a higher R-value, save fit
            % parameters otherwise sav surface 3-mode model fit parameters
            if round(ra,5) > round(rb,5)
                mdc3 = [mdl33(1) mdl22(1:2) mdl33(1) mdl22(3:4) mlc2];
                mdres_A2 = nan(1,12);
                jjk = 1;jkk = 3;jjk2 = 1;jkk2 = 3;
                for ijh = 1:length(akh)
                    if akh(ijh)  == 1
                        mdres_A2(jjk:jkk) = mdc3(jjk2:jkk2);
                        jjk2 = jjk2+3;jkk2 = jkk2+3;
                    end
                    jjk = jjk+3;jkk = jkk+3;
                end
                sd_param(1,:) = mdres_A2;
                sd_gof(1,:) = [ra,pa];
            else
                mdc3 = [mdl33(1) mdl22(1:2) mdl33(1) mdl22(3:4)];
                jjk2 = 1;jkk2 = 3;
                mdres_A2 = nan(1,12);
                jjk = 1;jkk = 3;
                for ijh = 1:length(akh)
                    if akh(ijh) == 1
                        mdres_A2(jjk:jkk) = mdc3(jjk2:jkk2);
                        jjk2 = jjk2+3;jkk2 = jkk2+3;
                    end
                    jjk = jjk+3;jkk = jkk+3;
                end
                sd_param(1,:) = mdres_A2;
                sd_gof(1,:) = [rb,pb];
            end
        end
        
        % if the 3-mode surface model has the highest (lowest) R (p-value),
        % proceed to fit 3-mode volume model
        if r(3) == max(r) && r(3) > 0.9 && p(3) < 0.01
            % define 2-mode volume model and initial bounds/starting points
            % of the coarse mode
            mdcb = [];rab = [];pab = [];
            m4fun = @(b,x)(2.203*pi*x.^3).*((mc4(1)./((sqrt(2*pi))*log( ...
                mc4(2)))).*exp(-((log(x)-log(mc4(3))).^2./(2*log(mc4(2) ...
                ).^2)))+b(1)./((sqrt(2*pi)).*log(b(2))).*exp(-((log(x) ...
                -log(b(3))).^2./(2*log(b(2)).^2))))/6;
            lb = [(10^(-9))*Nc(1) 1.20 900];
            sp = [(1)*Nc(1) 1.60 2500]*stp_fct(i6);
            ub = [(1.5)*Nc(1) 2.24 UB];
            
            % fit the 2-mode volume model using the initial bounds/starting
            % point and determine GOF
            md00 = real(lsqcurvefit(m4fun,sp,Dp, ...
                (pi*Dp.^3).*(dndlogD)/6,lb,ub,opts));
            [rab(1),pab(1)] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                m4fun(md00,Dp)');
            mdcb(1,:) = md00;
            
            % Initialize 2nd stage of fitting process by keeping the Nj
            % values fixed, and set the new bounds/starting points
            m4fun_2 = @(b,x)(2.203*pi*x.^3).*((mc4(1)./((sqrt(2*pi))* ...
                log(mc4(2)))).*exp(-((log(x)-log(mc4(3))).^2./(2*log( ...
                mc4(2)).^2)))+ md00(1)./((sqrt(2*pi)).*log(b(1))).*exp( ...
                -((log(x)-log(b(2))).^2./(2*log(b(1)).^2))))/6;
            lb12 = [md00(2) md00(3)]*acy(1);
            sp12 = [md00(2) md00(3)]*acy(2);
            ub12 = [md00(2) md00(3)]*acy(3);
            
            % fit the 2nd 2-mode volume model using the new bounds/starting
            % points and determine GOF
            mdl1c = real(lsqcurvefit(m4fun_2,sp12,Dp, ...
                (pi*Dp.^3).*(dndlogD)/6,lb12,ub12,opts));
            [rab(2),pab(2)] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                m4fun_2(mdl1c,Dp)');
            mdcb(2,:) = [md00(1) mdl1c];
            
            % Initialize 3rd stage of fitting process by keeping the sgj
            % and Dgj values fixed, and set the new bounds/starting points
            m4fun_3 = @(b,x)(2.203*pi*x.^3).*((mc4(1)./((sqrt(2*pi))* ...
                log(mc4(2)))).*exp(-((log(x)-log(mc4(3))).^2./(2*log( ...
                mc4(2)).^2)))+b(1)./((sqrt(2*pi)).*log(mdl1c(1))).*exp( ...
                -((log(x)-log(mdl1c(2))).^2./(2*log(mdl1c(1)).^2))))/6;
            lb11 = md00(1)*acy(1);sp11 = md00(1)*acy(2);
            ub11 = md00(1)*acy(3);
            
            % fit the 3rd 2-mode model using the new bounds/starting points
            % and determine GOF
            mdlc = real(lsqcurvefit(m4fun_3,sp11,Dp, ...
                (pi*Dp.^3).*(dndlogD)/6,lb11,ub11,opts));
            [rab(3),pab(3)] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                m4fun_3(mdlc,Dp'));
            mdcb(3,:) = [mdlc mdl1c];
            
            % find the 2-mode volume model that has the highest R and
            % p-value to carry forward for the final optimization step
            kji = find(rab == max(rab,[],'all','omitnan'));
            ra=rab(kji(end));pa=pab(kji(end));mlc2 = mdcb(kji(end),:);
            
            % determine GOF of 2-mode volume model using orginal 3-mode
            % model Nj, Dgj, and sgj
            [rb,pb] = corr((pi*Dp'.^3).*(dndlogD')/6, ...
                twomodefun_3(mdl13,Dp'));
            
            % if the volume 2-mode model has a higher R-value, save fit
            % parameters otherwise sav surface 3-mode model fit parameters
            if round(ra,5) >= round(rb,5)
                sd_param(1,:) = [mc4(1:3),nan(1,6),mlc2];
                sd_gof(1,:) = [ra,pa];
            else
                sd_param(1,:) = mc4;
                sd_gof(1,:) = [rb,pb];
            end
        end
        
        % Failsafe Pathway
        % if 3 modes are detected, fit a 3-mode surface model
    elseif length(ACo)==3
        
        % define 3-mode surface model and initial bounds/starting points
        mdcab = [];rab = [];pab = [];
        aa=~isnan(Nt(1,:));
        nn = Nt(1,ijo);
        lb = zeros(1,length(ijo)*3);sp = zeros(1,length(ijo)*3);
        ub = zeros(1,length(ijo)*3);
        jjk = 1;jkk = 3;
        for ikh = 1:length(ijo)
            lb(1,jjk:jkk) = [nn(ikh)*lb01(ijo(ikh)) lb0(ijo(ikh)) ...
                lb00(ijo(ikh))];
            sp(1,jjk:jkk) = [nn(ikh)*sp01(ijo(ikh)) sp0(ijo(ikh)) ...
                sp00(ijo(ikh))]*stp_fct(i6);
            ub(1,jjk:jkk) = [nn(ikh)*ub01(ijo(ikh)) ub0(ijo(ikh)) ...
                ub00(ijo(ikh))];
            jjk = jjk + 3;jkk = jkk + 3;
        end        
        threemodefun = @(b,x)(2.203*pi*x.^2).*((b(1)/((sqrt(2*pi))* ...
            log(b(2)))).*exp(-((log(x)-log(b(3))).^2./(2*log(b(2)).^2)))+...
            b(4)/((sqrt(2*pi)).*log(b(5))).*exp(-((log(x)-log(b(6)) ...
            ).^2./(2*log(b(5)).^2)))+b(7)/((sqrt(2*pi)).*log(b(8)) ...
            ).*exp(-((log(x)-log(b(9))).^2./(2*log(b(8)).^2))));
        
        % fit the 3-mode model using the initial bounds/starting point and
        % determine GOF
        mdl = real(lsqcurvefit(threemodefun,sp,Dp, ... 
            (pi*Dp.^2).*dndlogD,lb,ub,opts));
        [rab(1),pab(1)] = corr((pi*Dp'.^2).*dndlogD',threemodefun(mdl,Dp'));
        mdcab(1,:) = mdl;
        
        % Initialize 2nd stage of fitting process by keeping the Nj
        % values fixed, and set the new bounds/starting points
        threemodefun_2 = @(b,x)(2.203*pi*x.^2).*((nn(1)/((sqrt(2*pi) ...
            )*log(b(1)))).*exp(-((log(x)-log(b(2))).^2./(2*log(b(1)).^2)...
            ))+nn(2)/((sqrt(2*pi)).*log(b(3))).*exp(-((log(x)-log(b(4)) ...
            ).^2./(2*log(b(3)).^2)))+nn(3)/((sqrt(2*pi)).*log(b(5)) ...
            ).*exp(-((log(x)-log(b(6))).^2./(2*log(b(5)).^2))));
        lb2 = [mdl(2) mdl(3) mdl(5) mdl(6) mdl(8) mdl(9)]*acy(1);
        sp2 = [mdl(2) mdl(3) mdl(5) mdl(6) mdl(8) mdl(9)]*acy(2);
        ub2 = [mdl(2) mdl(3) mdl(5) mdl(6) mdl(8) mdl(9)]*acy(3);
        
        % fit the 2nd 3-mode model using the new bounds/starting points
        % and determine GOF
        mdl2 = real(lsqcurvefit(threemodefun_2,sp2,Dp, ...
            (pi*Dp.^2).*dndlogD,lb2,ub2,opts));
        [rab(2),pab(2)] = corr((pi*Dp'.^2).*dndlogD', ...
            threemodefun_2(mdl2,Dp'));
        mdcab(2,:) = [mdl(1) mdl2(1:2) mdl(4) mdl2(3:4) mdl(7) mdl2(5:6)];
        
        % Initialize 3rd stage of fitting process by keeping the sgj
        % and Dgj values fixed, and set the new bounds/starting points
        threemodefun_3 = @(b,x)(2.203*pi*x.^2).*((b(1)/((sqrt(2*pi) ...
            )*log(mdl2(1)))).*exp(-((log(x)-log(mdl2(2))).^2./(2*log( ... 
            mdl2(1)).^2)))+b(2)/((sqrt(2*pi)).*log(mdl2(3))).*exp(-(( ...
            log(x)-log(mdl2(4))).^2./(2*log(mdl2(3)).^2)))+b(3)/((sqrt( ...
            2*pi)).*log(mdl2(5))).*exp(-((log(x)-log(mdl2(6))).^2./ ... 
            (2*log(mdl2(5)).^2))));
        lb3 = mdl([1 4 7])*acy(1);sp3 = mdl([1 4 7])*acy(2);
        ub3 = mdl([1 4 7])*acy(3);
        
        % fit the 3rd 3-mode model using the new bounds/starting points
        % and determine GOF
        mdl3 = real(lsqcurvefit(threemodefun_3,sp3,Dp, ...
            (pi*Dp.^2).*dndlogD,lb3,ub3,opts));
        [rab(3),pab(3)] = corr((pi*Dp'.^2).*dndlogD', ...
            (threemodefun_3(mdl3,Dp')));
        mdcab(3,:) = [mdl3(1) mdl2(1:2) mdl3(2) mdl2(3:4) ...
            mdl3(3) mdl2(5:6)];
        
        % find the 3-mode model that has the highest R and p-value 
        kji = find(rab == max(rab,[],'all','omitnan'));
        r=rab(kji(end));p=pab(kji(end));mdc = mdcab(kji(end),:);
        
        % Save fit parameters
        jjk = 1;jkk = 3;jjk2 = 1;jkk2 = 3;
        mdres_A2 = nan(1,12);
        if r > 0.9 && p < 0.01
            for igu = 1:length(aa)
                if aa(igu)==1
                    mdres_A2(jjk:jkk) = mdc(jjk2:jkk2);
                    jjk2 = jjk2 + 3;jkk2 = jkk2 + 3;
                end
                jjk = jjk + 3;jkk = jkk + 3;
            end
            sd_param(1,:) = mdres_A2;
            sd_gof(1,:) = [r,p];
        end
        
        % if 2 modes are detected, fit a 2-mode surface model
    elseif length(ACo)==2
        
        % define 2-mode surface model and initial bounds/starting points
        mdcab = [];rab = [];pab = [];
        aa=~isnan(Nt(1,:));
        nn = Nt(1,ijo);
        lb = zeros(1,length(ijo)*3);sp = zeros(1,length(ijo)*3);
        ub = zeros(1,length(ijo)*3);
        jjk = 1;jkk = 3;
        for ikh = 1:length(ijo)
            lb(1,jjk:jkk) = [nn(ikh)*lb01(ijo(ikh)) lb0(ijo(ikh)) ...
                lb00(ijo(ikh))];
            sp(1,jjk:jkk) = [nn(ikh)*sp01(ijo(ikh)) sp0(ijo(ikh)) ...
                sp00(ijo(ikh))]*stp_fct(i6);
            ub(1,jjk:jkk) = [nn(ikh)*ub01(ijo(ikh)) ub0(ijo(ikh)) ...
                ub00(ijo(ikh))];
            jjk = jjk + 3;jkk = jkk + 3;
        end
        twomodefun = @(b,x)(2.203*pi*x.^2).*((b(1)/((sqrt(2*pi))*log( ...
            b(2)))).*exp(-((log(x)-log(b(3))).^2./(2*log(b(2)).^2)))+...
            b(4)/((sqrt(2*pi)).*log(b(5))).*exp(-((log(x)-log(b(6)) ...
            ).^2./(2*log(b(5)).^2))));
        
        % fit the 2-mode model using the initial bounds/starting point and
        % determine GOF
        mdl = real(lsqcurvefit(twomodefun,sp,Dp, ...
            (pi*Dp.^2).*dndlogD,lb,ub,opts));
        [rab(1),pab(1)] = corr((pi*Dp'.^2).*dndlogD', ...
            twomodefun(mdl,Dp'));
        mdcab(1,:) = mdl;
        
        % Initialize 2nd stage of fitting process by keeping the Nj
        % values fixed, and set the new bounds/starting points
        twomodefun_2 = @(b,x)(2.203*pi*x.^2).*((nn(1)/((sqrt(2*pi))*log( ...
            b(1)))).*exp(-((log(x)-log(b(2))).^2./(2*log(b(1)).^2)))+...
            (nn(2))/((sqrt(2*pi)).*log(b(3))).*exp(-((log(x)-log(b(4)) ...
            ).^2./(2*log(b(3)).^2))));
        lb2 = [mdl(2) mdl(3) mdl(5) mdl(6)]*acy(1);
        sp2 = [mdl(2) mdl(3) mdl(5) mdl(6)]*acy(2);
        ub2 = [mdl(2) mdl(3) mdl(5) mdl(6)]*acy(3);
        
        % fit the 2nd 2-mode model using the new bounds/starting points
        % and determine GOF
        mdl2 = real(lsqcurvefit(twomodefun_2,sp2,Dp, ...
            (pi*Dp.^2).*dndlogD,lb2,ub2,opts));
        [rab(2),pab(2)] = corr((pi*Dp'.^2).*dndlogD', ...
            twomodefun_2(mdl2,Dp'));
        mdcab(2,:) = [mdl(1) mdl2(1:2) mdl(4) mdl2(3:4)];
        
        % Initialize 3rd stage of fitting process by keeping the sgj
        % and Dgj values fixed, and set the new bounds/starting points        
        twomodefun_3 = @(b,x)(2.203*pi*x.^2).*((b(1)/((sqrt(2*pi))*log( ...
            mdl2(1)))).*exp(-((log(x)-log(mdl2(2))).^2./(2*log(mdl2(1) ...
            ).^2)))+(b(2))/((sqrt(2*pi)).*log(mdl2(3))).*exp(-((log(x)- ...
            log(mdl2(4))).^2./(2*log(mdl2(3)).^2))));
        lb3 = mdl([1 4])*acy(1);sp3 = mdl([1 4])*acy(2);
        ub3 = mdl([1 4])*acy(3);
        
        % fit the 3rd 2-mode model using the new bounds/starting points
        % and determine GOF       
        mdl3 = real(lsqcurvefit(twomodefun_3,sp3,Dp, ...
            (pi*Dp.^2).*dndlogD,lb3,ub3,opts));
        [rab(3),pab(3)] = corr((pi*Dp'.^2).*dndlogD', ...
            (twomodefun_3(mdl3,Dp')));
        mdcab(3,:) = [mdl3(1) mdl2(1:2) mdl3(2) mdl2(3:4)];
        
        % find the 2-mode model that has the highest R and p-value 
        kji = find(rab == max(rab,[],'all','omitnan'));
        r=rab(kji(end));p=pab(kji(end));mdc = mdcab(kji(end),:);
        
        % Save fit parameters
        jjk = 1;jkk = 3;jjk2 = 1;jkk2 = 3;
        mdres_A2 = nan(1,12);
        if r > 0.9 && p < 0.01
            for igu = 1:length(aa)
                if aa(igu)==1
                    mdres_A2(jjk:jkk) = mdc(jjk2:jkk2);
                    jjk2 = jjk2 + 3;jkk2 = jkk2 + 3;
                end
                jjk = jjk + 3;jkk = jkk + 3;
            end
            sd_param(1,:) = mdres_A2;
            sd_gof(1,:) = [r,p];
        end
        
        % if 1 mode is detected, fit a 1-mode surface model
    elseif length(ACo)==1
        if length(Dp)>3
           
            % define 1-mode surface model and initial bounds/starting points
            mdcab = [];rab = [];pab = [];
            aa=~isnan(Nt(1,:));
            nn = Nt(1,ijo);
            lb = zeros(1,length(ijo)*3);sp = zeros(1,length(ijo)*3);
            ub = zeros(1,length(ijo)*3);
            jjk = 1;jkk = 3;
            for ikh = 1:length(ijo)
                lb(1,jjk:jkk) = [nn(ikh)*lb01(ijo(ikh)) lb0(ijo(ikh)) ...
                    lb00(ijo(ikh))];
                sp(1,jjk:jkk) = [nn(ikh)*sp01(ijo(ikh)) sp0(ijo(ikh)) ...
                    sp00(ijo(ikh))]*stp_fct(i6);
                ub(1,jjk:jkk) = [nn(ikh)*ub01(ijo(ikh)) ub0(ijo(ikh)) ...
                    ub00(ijo(ikh))];
                jjk = jjk + 3;jkk = jkk + 3;
            end
            onemodefun = @(b,x)(2.203*pi*x.^2).*((b(1)/((sqrt(2*pi) ...
                )*log(b(2)))).*exp(-((log(x)-log(b(3))).^2./(2*log(b(2) ...
                ).^2))));
            
            % fit the 1-mode model using the initial bounds/starting point 
            % and determine GOF
            mdl = real(lsqcurvefit(onemodefun,sp,Dp, ...
                (pi*Dp.^2).*dndlogD,lb,ub,opts));
            [rab(1),pab(1)] = corr((pi*Dp'.^2).*dndlogD', ...
                onemodefun(mdl,Dp'));
            mdcab(1,:) = mdl;
            
            % Initialize 2nd stage of fitting process by keeping the Nj
            % values fixed, and set the new bounds/starting points            
            onemodefun_2 = @(b,x)((2.203*pi*x.^2).*(nn(1)/((sqrt(2*pi) ...
                )*log(b(1)))).*exp(-((log(x)-log(b(2))).^2./(2*log(b(1) ...
                ).^2))));
            lb2 = [mdl(2) mdl(3)]*acy(1);
            sp2 = [mdl(2) mdl(3)]*acy(2);
            ub2 = [mdl(2) mdl(3)]*acy(3);
            
        
            % fit the 2nd 1-mode model using the new bounds/starting points
            % and determine GOF            
            mdl2 = real(lsqcurvefit(onemodefun_2,sp2,Dp, ...
                (pi*Dp.^2).*dndlogD,lb2,ub2,opts));
            [rab(2),pab(2)] = corr((pi*Dp'.^2).*dndlogD', ...
                onemodefun_2(mdl2,Dp'));
            mdcab(2,:) = [mdl(1) mdl2(1:2)];
            
            % Initialize 3rd stage of fitting process by keeping the sgj
            % and Dgj values fixed, and set the new bounds/starting points             
            onemodefun_3 = @(b,x)(2.203*pi*x.^2).*((b(1)/((sqrt(2*pi) ...
                )*log(mdl2(1)))).*exp(-((log(x)-log(mdl2(2))).^2./( ...
                2*log(mdl2(1)).^2))));
            lb3 = mdl(1)*acy(1);sp3 = mdl(1)*acy(2);ub3 = mdl(1)*acy(3);
            
            % fit the 3rd 1-mode model using the new bounds/starting points
            % and determine GOF
            mdl3 = real(lsqcurvefit(onemodefun_3,sp3,Dp, ...
                (pi*Dp.^2).*dndlogD,lb3,ub3,opts));
            [rab(3),pab(3)] = corr((pi*Dp'.^2).*dndlogD', ...
                (onemodefun_3(mdl3,Dp')));
            mdcab(3,:) = [mdl3(1) mdl2(1:2)];
           
            % find the 2-mode model that has the highest R and p-value 
            kji = find(rab == max(rab,[],'all','omitnan'));
            r=rab(kji(end));p=pab(kji(end));mdc = mdcab(kji(end),:);
            
            % Save fit parameters
            jjk = 1;jkk = 3;jjk2 = 1;jkk2 = 3;
            mdres_A2 = nan(1,12);
            if r > 0.9 && p < 0.01
                for igu = 1:length(aa)
                    if aa(igu)==1
                        mdres_A2(jjk:jkk) = mdc(jjk2:jkk2);
                        jjk2 = jjk2 + 3;jkk2 = jkk2 + 3;
                    end
                    jjk = jjk + 3;jkk = jkk + 3;
                end
                sd_param(1,:) = mdres_A2;
                sd_gof(1,:) = [r,p];
            end
        end
    end
    % if a solution is found, do not repeat fitting process with a new
    % starting point
    if ~isnan(sd_gof(1,1))
        break
    end
end
% format and export output
op = [sd_param,sd_gof];
op.Properties.VariableNames = ["N1","sg1","Dg1","N2","sg2","Dg2","N3","sg3",...
    "Dg3","Nc","sgc","Dgc","R","pvalue"];
end
