%Retrieve Total Dry Refractive Index and Hygroscopicity 
function op = m_g_ret(n_o,D,mR,mI,Cs,Ca,Cws,Lmda_s,Lmda_a,Lmda_w_s,RH,Qf,...
    Acur,Uncr,npt,cFct)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following code is used to retrieve both total dry complex refractive
% index (i.e., m_d,t = n_d,t + k_d,t) and hygroscopicity (i.e., kappa).
% Calculated total ambient extinction coefficient at 550 nm (i.e., Cwe) and
% retrieved total growth factor (i.e., g) at the given relative humidity
% (RH) are also provided part of the of the final output (i.e., op). This
% code requires the set of Mie forward model MATLAB functions (C. Mätzler
% May, 2002) which was retrieved from https://omlc.org/software/mie/ on
% January first, 2019 and is available with this work. The "S_e.m" function
% to generate the covariance matrix and "MieKernal.m" must be used to
% generate the dry scattering and absorption efficiency lookup table (i.e.,
% Qf) for the data set. Each of the required inputs for this code are
% described as follows:
% 1) n_o: Measured lognormal size resolved number concentration.
%    -> dimensions: [length of data set, number of measurement channels]
%    -> Units: # cm-3
% 2) D: The n_o measurement channels' mid-point diameters.
%    -> dimensions: [number of no measurement channels,1]
%    -> Units: nm
% 3) mR: Ascending column vector of desired search range of the real
%    portion of the m_d,t (i.e., n_d,t).
%    -> dimensions: [1, number of mR increments]
%    -> EXAPMLE: 1.33:0.01:1.70
%    -> Units: unitless
% 4) mI: Ascending column vector of desired search range of the imaginary
%    portion of the m_d,t (i.e., k_d,t).
%    -> dimensions: [1, number of mI increments]
%    -> EXAPMLE: 0:0.01:0.1
%    -> Units: unitless
% 5) Cs: Dry scattering coefficients measured in 3 wavelengths.
%    -> dimensions: [length of data set, 3]
%    -> Units: Mm-1
% 6) Ca: Dry absorption coefficients measured in 3 wavelengths.
%    -> dimensions: [length of data set, 3]
%    -> Units: Mm-1
% 7) Cws: Humidified scattering coefficients measured in 1 wavelength.
%    -> dimensions: [length of data set, 1]
%    -> Units: Mm-1
% 8) Lmda_s: Wavelengths that correspond to the 3 dry scattering
%    coefficient measurement channels.
%    -> dimensions: [1, 3]
%    -> Units: nm
% 9) Lmda_a: Wavelengths that correspond to the 3 dry absorption
%    coefficient measurement channels.
%    -> dimensions: [1, 3]
%    -> Units: nm
% 10) Lmda_w_s: Wavelength that corresponds to the humidified scattering
%     coefficient measurement channels.
%     -> dimensions: [1, 1]
%     -> Units: nm
% 11) RH: Measured relative humidity.
%     -> dimensions: [length of data set,1]
%     -> Units: nm
% 12) Qf: Dry scattering and absorption efficiency lookup table for the 3
%     scattering and 3 absorption wavelengths measured.
%     -> dimensions: [number of mR increments, number of mI increments,
%         number of no measurement channels,6]
%     -> Units: unitless
% 13) Acur: 3 Scattering and 3 absorption measurement accuracy values used
%     to calculate Chi'.
%     -> dimensions: [1, 6]
%     -> FORMAT: [accuracy of scattering channels 1 to 3, accuracy of
%                 absorption channels 1 to 3]
%     -> EXAPMLE: [0.1, 0.1, 0.1, 0.05, 0.05, 0.05]
%     -> Units: unitless
% 14) Uncr: 3 Scattering and 3 absorption measurement precision values
%     used to calculate Chi'.
%     -> dimensions: [1, 6]
%     -> FORMAT: [precision of scattering channels 1 to 3, precision of
%                 absorption channels 1 to 3]
%     -> Units: Mm-1
% 15) npt: Resolution of data set (i.e. number of native optical
%     measurements averaged to create data set). Used to calculate Chi'.
%     -> dimensions: [1, 1]
%     -> EXAPMLE: (10 second)/(1 second)
%     -> Units: unitless
% 16) cFct: Optical measurement correlation factor used to calculate Chi'.
%     -> dimensions: [1, 1]
%     -> EXAPMLE: 0.3
%     -> Units: unitless
% The output of this code is op, which is a MATLAB structure that is
% described as follows:
% 1) op.DryRetrievals: Table containing the average of at least 10 valid
%    n_d,t solutions, average of at least 10 valid n_d,t solutions,
%    absolute relative error in 3 scattering channels, absolute error in
%    absorption channels, and Chi' threshold corresponding to where 10 or
%    more solutions are valid.
%    -> dimensions: [length of data set,9]
%    -> Units: [unitless, unitless, unitless, 
%               unitless, unitless, Mm-1, Mm-1, Mm-1, unitless]
% 2) op.WetRetrievals: Table containing the first valid kappa solution,
%    the first valid g solution, and the Cwe at this solution.
%    -> dimensions: [length of data set,3]
%    -> Units: [unitless, unitless, Mm-1]
% *Designed for MATLAB 2021a and requires the machine learning, statistics
% and parallel computing MATLAB toolboxes*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
% expected hygroscopicity range for atmospheric aerosol particles
kappa_rng = [0 1.4];
 
% create column array of Chi' thresholds for m_d,t retrievals
thrsh = 0.1:0.1:1.0;
 
% relative error threshold used for the retrieval of kappa
th2 = 0.01;
 
% calculate the standard deviation in measurements from Uncr and npt
sig1 = Uncr./sqrt(npt);
 
% number of points required to consider a set of solutions valid for
% averaging
tpts = 10;
 
% reduce dimensions and match dimensions of the Qf
wvl = [Lmda_s,Lmda_a];
B_meas = [Cs,Ca];
 
% identify the Cs channel that corresponds to Cws
io = find(Lmda_s == Lmda_w_s);
ScatDry = Cs(:,io);
 
% initialize the computation of the full dry optical coefficient solution
% space (i.e., Cf)
L1 = length(mR);L2 = length(mI);L3 = length(wvl);L4 = size(n_o,1);
Cf = nan(size(n_o,1),length(mR),length(mI),length(wvl));
 
% remove nan values to prevent "trapz.m" from failing
n_o(isnan(n_o)) = 0;
 
% compute Cf
for h = 1:L3
    parfor i=1:L1
        for j=1:L2
            Cf(:,i,j,h) = (pi/4)*(trapz(log10(D),(D.^2).*...
                squeeze(Qf(:,i,j,h)).*n_o',1))*1e-6; %Mm^-1
        end
    end
end
% replace nan values
n_o(n_o ==0) = nan;
 
% initialize and solve iteratively for mR and mI
retrieval = nan(size(n_o,1),9);
for i3=1:L4
    dNdlogD = n_o(i3,:);
    if length(find(~isnan(dNdlogD)))>2
        delta = nan(length(mR),length(mI),size(B_meas,2));
        idx = nan(length(mR),length(mI),2);
        ch2 = nan(length(mR),length(mI),1);
        b_measured = B_meas(i3,:)';CF = squeeze(Cf(i3,:,:,:));
        parfor mr=1:L1
            for mi = 1:L2
                % difference between the measured and the calculated
                % scattering and absorption coefficients.
                delta(mr,mi,:) = (b_measured-squeeze(CF(mr,mi,:)));
                
                % calculate the covariance matrix
                covM = inv((SE(b_measured,Acur,sig1,cFct).^(2)));
                
                % calculate each position of chi
                ch2(mr,mi,:) = sqrt(0.5.*((squeeze(delta(mr,mi,:)))'*...
                    (covM)*squeeze(delta(mr,mi,:))))/6;
                
                % matrix position indicator
                idx(mr,mi,:) = [mr mi];
            end
        end
        
        % reshape data and find possible valid solutions using the first Chi'
        % threshold
        idx2 = reshape(idx,[],2);
        delta2 = reshape(delta,[],size(B_meas,2));
        ch = reshape(ch2,[],1);
        teste2 = find(ch<=thrsh(1));
        
        % if not enough valid solutions are found, increase the Chi'
        % threshold value until the set includes more than 10 solutions
        if ~isempty(teste2) && length(teste2)>tpts
            in = 1;
        else
            for ik = 1:size(thrsh,2)
                teste2 = find(ch<=thrsh(ik));
                if length(teste2)>tpts
                    in = ik;
                    break
                end
            end
        end
        
        % as long as the final set of valid solutions contains more than 10
        % points, take the average of the valid and gather metadata for the
        % solution space
        if ~isempty(teste2)&& length(teste2)>tpts
            sol2 = [mR(idx2(teste2,1))' mI(idx2(teste2,2))'];
            d3 = [abs(delta2(teste2,[1 3 5]))./b_measured([1 3 5])',...
                abs(delta2(teste2,[2 4 6]))];
            retrieval(i3,:) = [mean([sol2 d3],1) in];
        end
    end
end
 
% save the all of m_d,t values retrieved from the data set
op.DryRetrievals =  array2table(retrieval);
op.DryRetrievals.Properties.VariableNames = ["n_d,t","k_d,t","k_d,t",...
    "AbsRErr_Cst_Ch1","AbsRErr_Cst_Ch2","AbsRErr_Cst_Ch3",...
    "AbsErr_Cat_Ch1","AbsErr_Cat_Ch2","AbsErr_Cat_Ch3"];
 
% initialize the retrieval of each kappa value from the data set
g = nan(size(retrieval,1),1);kappa = nan(size(retrieval,1),1);
Cwe_calc = nan(size(retrieval,1),1);
 
% define the growth factor space range of each retrieval using the relative
% humidity values and kapp_rng
gf1 = [real((kappa_rng(1).*RH./(100-RH)+1).^(1/3)) real((kappa_rng(2).*...
    RH./(100-RH)+1).^(1/3))];
 
% iterate through each data point in the set and perform the kappa
% retrieval
for i2=1:size(retrieval,1)
    dCs = ScatDry(i2);wCs = Cws(i2);
    
    % exclude nan values
    aio = find(~isnan(n_o(i2,:)));
    
    % initialize iterative solving of kappa while ensuring all required
    % measurements are available
    dndlogd = n_o(i2,aio)';
    DP = D(aio);
    if ~isnan(RH(i2))
        
        % create g solution space with increments of 10-3
        gf = gf1(i2,1):0.001:gf1(i2,2);
        retrA = retrieval(i2,1);retrB = retrieval(i2,2);
        if ~isnan(retrA) & ~isnan(retrB) & ~isnan(wCs)
            
            % if humidified scattering coefficient (Cws) is less than the
            % dry scattering coefficient, set kappa to 0
            if wCs > dCs
                sCw_recalc = nan(1,length(gf));eCw = nan(1,length(gf));
                delta = nan(1,length(gf));CRI_mR = delta;CRI_mI = delta;
                parfor j=1:length(gf)

                    % compute ambient complex refractive index using the
                    % m_d,t retrieved in the last step, the refractive
                    % index of water (Hale & Querry, 1973; Nessler et al.,
                    % 2005).
                    CRI_mR(j) = (retrA+1.33*((gf(j)^3)-1))/(gf(j)^3);
                    CRI_mI(j) = retrB/(gf(j)^3);
                    
                    % compute size parameter (i.e., x) for particle
                    % diameters shifted by each g space increment
                    x = pi*DP*gf(j)/Lmda_w_s;
                    % initialize and solve for humidified scattering
                    % efficiency
                    Qws = zeros(length(x),2);
                    for k = 1:length(x)
                        aio = Mie(complex(CRI_mR(j),CRI_mI(j)),x(k,1));
                        Qws(k,:) = aio(1:2);
                    end
                    
                    % compute Cws, Cwe and difference in calculated and
                    % measured Cws (i.e., delta)
                    sCw_recalc(j) = (pi/4)*(trapz(log10(DP*gf(j)),...
                        ((DP*gf(j)).^2).*Qws(:,2).*dndlogd))*1e-6; %Mm^-1
                    eCw(j) = (pi/4)*(trapz(log10(DP*gf(j)),...
                        (DP*gf(j).^2).*Qws(:,1).*dndlogd))*1e-6; %Mm^-1
                    delta(j) = sCw_recalc(j)-wCs;
                end
            end
            
            % if Cws is less than or equal to Cs, Cwe == Ce. Else, find the
            % first value in the g solution space where relative difference
            % between measured and calculated Cws is greater than 1%
            if wCs <= dCs
                jj1 = 1;
                CRI_mR = retrA;
                CRI_mI = retrB;
                x = pi*DP*gf(1)/Lmda_w_s;
                Qws = zeros(length(x),2);
                for k = 1:length(x)
                    aio = Mie(complex(CRI_mR(1),CRI_mI(1)),x(k,1));
                    Qws(k,:) = aio(1:2);
                end
                eCw = (pi/4)*(trapz(log10(DP*gf(1)),...
                    (DP*gf(1).^2).*Qws(:,1).*dndlogd))*1e-6;
            else
                jj1 = find(delta/wCs>=th2);
            end
            
            % if a valid solution is found for this data point, save the
            % valid g, kappa, and calculated (Cwe) for this data point.
            if ~isempty(jj1)
                g(i2) = gf((jj1(1)));
                kappa(i2) = real((g(i2).^3-1).*(100-RH(i2))/RH(i2));
                Cwe_calc(i2) = eCw((jj1(1)));
            end
        end
    end
end
 
% save g, kappa, and Cwe_calc retrieved for the data set
op.WetRetrievals = [g,kappa,Cwe_calc];
end